package config

import (
	model2 "ArmedPolice/app/common/model"
	"ArmedPolice/app/controller/basic"
	"ArmedPolice/app/model"
	"ArmedPolice/app/service"
	"ArmedPolice/config"
	"time"
)

type Breakdown struct{ *service.Session }

type BreakdownHandle func(session *service.Session) *Breakdown

type BreakdownInfo struct {
	basic.CommonIDString
	*model2.SysBreakdown
}

// List 列表信息
func (c *Breakdown) List(title string, page, pageSize int) (*basic.PageDataResponse, error) {
	mSysBreakdown := model.NewSysBreakdown()

	out := make([]*model2.SysBreakdown, 0)

	where := &model2.ModelWhereOrder{
		Order: model2.NewOrder("id", model2.OrderModeToDesc),
	}
	if title != "" {
		where.Where = model2.NewWhereLike("title", title)
	}
	var count int64

	if err := model2.Pages(mSysBreakdown.SysBreakdown, &out, page, pageSize, &count, where); err != nil {
		return nil, err
	}
	list := make([]*BreakdownInfo, 0)

	for _, v := range out {
		list = append(list, &BreakdownInfo{
			CommonIDString: basic.CommonIDString{ID: v.GetEncodeID()},
			SysBreakdown:   v,
		})
	}
	return &basic.PageDataResponse{Data: list, Count: count}, nil
}

// Select 查询信息
func (c *Breakdown) Select() ([]*BreakdownInfo, error) {
	mSysBreakdown := model.NewSysBreakdown()

	out := make([]*model2.SysBreakdown, 0)

	if err := model2.ScanFields(mSysBreakdown.SysBreakdown, &out, []string{"id", "title"}); err != nil {
		return nil, err
	}
	list := make([]*BreakdownInfo, 0)

	for _, v := range out {
		list = append(list, &BreakdownInfo{
			CommonIDString: basic.CommonIDString{ID: v.GetEncodeID()},
			SysBreakdown:   v,
		})
	}
	return list, nil
}

// Form 数据处理
func (c *Breakdown) Form(id uint64, title, remark string) error {
	mSysBreakdown := model.NewSysBreakdown()

	if id > 0 {
		mSysBreakdown.ID = id

		if err := model2.Updates(mSysBreakdown.SysBreakdown, map[string]interface{}{
			"title": title, "remark": remark, "updated_at": time.Now(),
		}); err != nil {
			return err
		}
	} else {
		mSysBreakdown.Title = title
		mSysBreakdown.Remark = remark

		if err := model2.Create(mSysBreakdown.SysBreakdown); err != nil {
			return err
		}
	}
	config.SystemBreakdown[id] = title
	return nil
}

// Delete 删除操作
func (c *Breakdown) Delete(id uint64) error {
	mSysBreakdown := model.NewSysBreakdown()
	mSysBreakdown.ID = id
	return model2.Delete(mSysBreakdown.SysBreakdown)
}

func NewBreakdown() BreakdownHandle {
	return func(session *service.Session) *Breakdown {
		return &Breakdown{session}
	}
}
