package role

import (
	model2 "ArmedPolice/app/common/model"
	"ArmedPolice/app/controller/menu"
	"ArmedPolice/app/model"
	"ArmedPolice/app/service"
	"ArmedPolice/serve/orm"
	"errors"
	"gorm.io/gorm"
)

type Menu struct{ *service.Session }

type MenuHandle func(session *service.Session) *Menu

// List 菜单列表
func (c *Menu) List(roleID uint64) ([]*menu.InstanceIdentityInfo, error) {
	mSysMenu := model.NewSysMenu()
	out, err := mSysMenu.RoleMenu(roleID)

	if err != nil {
		return nil, err
	}
	return menu.TreeIdentity(mSysMenu.SysMenu, out, 0), nil
}

// Bind 绑定菜单
func (c *Menu) Bind(roleID uint64, menuIDs []uint64) error {
	if len(menuIDs) > 0 {
		var count int64

		mSysMenu := model.NewSysMenu()

		if err := model2.Count(mSysMenu.SysMenu, &count, model2.NewWhereIn("id", menuIDs),
			model2.NewWhere("auth", model2.SysMenuAuthForSystem)); err != nil {
			return err
		} else if count > 0 {
			return errors.New("不可设置超管菜单")
		}
	}
	mSysRoleMenu := model.NewSysRoleMenu()

	return orm.GetDB().Transaction(func(tx *gorm.DB) error {
		err := model2.DeleteWhere(mSysRoleMenu.SysRoleMenu, []*model2.ModelWhere{model2.NewWhere("role_id", roleID)}, tx)

		if err != nil {
			return err
		}
		if len(menuIDs) <= 0 {
			return nil
		}
		menus := make([]*model2.SysRoleMenu, 0)

		mark := make(map[uint64]uint64, 0)

		for _, v := range menuIDs {
			if _, has := mark[v]; has {
				continue
			}
			menus = append(menus, &model2.SysRoleMenu{
				RoleID: roleID, MenuID: v,
			})
			mark[v] = v
		}
		if err = model2.Creates(mSysRoleMenu.SysRoleMenu, menus, tx); err != nil {
			return err
		}
		return nil
	})
}

func NewMenu() MenuHandle {
	return func(session *service.Session) *Menu {
		return &Menu{session}
	}
}
