package model

import (
	"ArmedPolice/app/common/model"
	"ArmedPolice/serve/orm"
	"fmt"
)

type SysMenu struct {
	*model.SysMenu
}

type SysMenuScene struct {
	ID uint64 `json:"id"`
	*model.SysMenuBasic
	SceneID uint64 `json:"scene_id"` // 场景（租户，角色）
}

// Menus 菜单信息
func (m *SysMenu) Menus() ([]*model.SysMenu, error) {
	out := make([]*model.SysMenu, 0)

	db := orm.GetDB().Table(m.TableName()).Where("is_deleted = ?", model.DeleteStatusForNot)

	if err := db.Order("parent_id " + model.OrderModeToAsc).Order("sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// SystemMenu 系统菜单
func (m *SysMenu) SystemMenu() ([]*SysMenuScene, error) {
	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(m.TableName()).
		Select("id, parent_id, name, kind, link, component, icon, 1 AS scene_id").
		Where("status = ? AND is_deleted = ?", model.SysMenuStatusForNormal, model.DeleteStatusForNot)

	if err := db.Order("parent_id " + model.OrderModeToAsc).Order("sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// UserMenu 用户菜单
func (m *SysMenu) UserMenu(uid uint64) ([]*SysMenuScene, error) {
	mSysUserRole := NewSysUserRole()

	mSysRoleMenu := NewSysRoleMenu()

	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(mSysUserRole.TableName()+" AS u_r").
		Select("m.id, m.parent_id, m.name, m.kind, m.link, m.component, m.icon, 1 AS scene_id").
		Joins(fmt.Sprintf("LEFT JOIN %s AS r_m ON u_r.role_id = r_m.role_id AND r_m.is_deleted = %d",
			mSysRoleMenu.TableName(), model.DeleteStatusForNot)).
		Joins(fmt.Sprintf("LEFT JOIN %s AS m ON r_m.menu_id = m.id AND m.status = %d AND m.is_deleted = %d",
			m.TableName(), model.SysMenuStatusForNormal, model.DeleteStatusForNot)).
		Where("u_r.uid = ? AND u_r.is_deleted = ?", uid, model.DeleteStatusForNot)

	if err := db.Order("m.parent_id " + model.OrderModeToAsc).Order("m.sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

// RoleMenu 角色菜单
func (m *SysMenu) RoleMenu(roleID uint64) ([]*SysMenuScene, error) {
	mSysRoleMenu := NewSysRoleMenu()

	out := make([]*SysMenuScene, 0)

	db := orm.GetDB().Table(m.TableName()+" AS m").
		Select("m.id, m.parent_id, m.name, m.kind, m.link, m.component, m.icon, r_m.id AS scene_id").
		Joins(fmt.Sprintf("LEFT JOIN %s AS r_m ON m.id = r_m.menu_id AND r_m.role_id = %d AND r_m.is_deleted = %d",
			mSysRoleMenu.TableName(), roleID, model.DeleteStatusForNot)).
		Where("m.status = ? AND m.is_deleted = ?", model.SysMenuStatusForNormal, model.DeleteStatusForNot)

	if err := db.Order("m.parent_id " + model.OrderModeToAsc).Order("m.sort " + model.OrderModeToDesc).Scan(&out).Error; err != nil {
		return nil, err
	}
	return out, nil
}

func NewSysMenu() *SysMenu {
	return &SysMenu{SysMenu: model.NewSysMenu()}
}
