package model

import (
	"ArmedPolice/app/common/model"
	"ArmedPolice/serve/orm"
	"fmt"
)

type WorkPurchase struct {
	*model.WorkPurchase
}

type WorkPurchaseInfo struct {
	*model.WorkPurchase
	Price            float64 `json:"price"`
	MaterialNumber   float64 `json:"material_number"`
	MaterialCode     string  `json:"material_code"`
	MaterialTitle    string  `json:"material_title"`
	ManufacturerName string  `json:"manufacturer_name"`
	SupplierName     string  `json:"supplier_name"`
}

// Purchases 采购信息
func (m *WorkPurchase) Purchases(workID uint64) ([]*WorkPurchaseInfo, error) {
	mManageSupplier := model.NewManageSupplier()

	db := orm.GetDB().Table(m.TableName()+" AS p").
		Select("p.id", "m_p.price", "m_p.number AS material_number", "m.code AS material_code", "m.title AS material_title",
			"m_s.name AS manufacturer_name", "s.name AS supplier_id", "s.name AS supplier_name").
		Joins(fmt.Sprintf("LEFT JOIN %s AS m_p ON p.material_purchase_id = m_p.id", model.NewManageMaterialPurchase().TableName())).
		Joins(fmt.Sprintf("LEFT JOIN %s AS m ON m_p.material_id = m.id", model.NewManageMaterial().TableName())).
		Joins(fmt.Sprintf("LEFT JOIN %s AS s ON m_p.supplier_id = s.id", mManageSupplier.TableName())).
		Joins(fmt.Sprintf("LEFT JOIN %s AS m_s ON m.manufacturer_id = m_s.id", mManageSupplier.TableName())).
		Where("p.work_id = ?", workID).
		Where("p.is_deleted = ? AND m_p.is_deleted = ?", model.DeleteStatusForNot, model.DeleteStatusForNot)

	out := make([]*WorkPurchaseInfo, 0)

	err := db.Scan(&out).Error

	return out, err
}

func NewWorkPurchase() *WorkPurchase {
	return &WorkPurchase{model.NewWorkPurchase()}
}
