package serve

import (
	"ArmedPolice/app"
	"ArmedPolice/app/common"
	"ArmedPolice/config"
	"ArmedPolice/router"
	"ArmedPolice/serve/cache"
	"ArmedPolice/serve/logger"
	"ArmedPolice/serve/orm"
	"ArmedPolice/serve/web"
	"ArmedPolice/task"
	"ArmedPolice/utils"
	"github.com/gin-gonic/gin"
	"strings"
)

type Serve struct {
	*Option
}

type Option struct {
	Config string `json:"config"`
}

func (this *Serve) Run() {
	// 载入配置
	utils.LoadConfig(this.Option.Config, config.SettingInfo, func(i interface{}) {
		obj := i.(*config.Setting)
		obj.Upload.Exts = strings.Split(obj.Upload.Ext, ",")
		logger.NewLogger().Init(&logger.Option{File: obj.Log.File, LeastDay: obj.Log.LeastDay, Level: obj.Log.Level, IsStdout: false}).Load()
	})
	orm.Init()
	common.Init()
	cache.Init()
	task.Init()
	app.Init()
	// 开启web
	web.NewWeb()(&web.WebConfig{
		Port: config.SettingInfo.Server.Port, ReadTimeout: config.SettingInfo.Server.ReadTimeout,
		WriteTimeout: config.SettingInfo.Server.WriteTimeout, IdleTimeout: config.SettingInfo.Server.IdleTimeout,
	}).Run(router.NewRouter(&router.Option{
		Mode: gin.DebugMode, IsCors: true,
		RateLimitConfig: &router.RateLimitConfig{
			IsRate: true, Limit: config.SettingInfo.Rate.Limit, Capacity: config.SettingInfo.Rate.Capacity,
		},
	}).Init())
}

func NewServe() func(option *Option) *Serve {
	return func(option *Option) *Serve {
		return &Serve{option}
	}
}
